@@ -14,15 +14,17 @@
 #' @param samplesize numeric. How many prediction samples should be used?
 #' @param sampling character. How to draw prediction samples? See \link[sp]{spsample}. Use sampling = "Fibonacci" for global applications.
 #' @param variables character vector defining the predictor variables used if type="feature. If not provided all variables included in modeldomain are used.
+#' @param timevar optional. character. Column that indicates the date. Only used if type="time".
+#' @param time_unit optional. Character. Unit for temporal distances See ?difftime.Only used if type="time".
 #' @return A data.frame containing the distances. Unit of returned geographic distances is meters. attributes contain W statistic between prediction area and either sample data, CV folds or test data. See details.
 #' @details The modeldomain is a sf polygon or a raster that defines the prediction area. The function takes a regular point sample (amount defined by samplesize) from the spatial extent.
 #'     If type = "feature", the argument modeldomain (and if provided then also the testdata and/or preddata) has to include predictors. Predictor values for x, testdata and preddata are optional if modeldomain is a raster.
-#'     If not provided they are extracted from the modeldomain rasterStack.
+#'     If not provided they are extracted from the modeldomain rasterStack. If some predictors are categorical (i.e., of class factor or character), gower distances will be used.
 #'     W statistic describes the match between the distributions. See Linnenbrink et al (2023) for further details.
 #' @note See Meyer and Pebesma (2022) for an application of this plotting function
 #' @seealso \code{\link{nndm}} \code{\link{knndm}}
 #' @import ggplot2
-#' @author Hanna Meyer, Edzer Pebesma, Marvin Ludwig
+#' @author Hanna Meyer, Edzer Pebesma, Marvin Ludwig, Jan Linnenbrink
 #' @examples
 #' \dontrun{
 #' library(CAST)
@@ -37,7 +39,10 @@
 #'
 #' ########### Distance between training data and new data:
 #' dist <- geodist(splotdata, studyArea)
+#' # With density functions
 #' plot(dist)
+#' # Or ECDFs (relevant for nndm and knnmd methods)
+#' plot(dist, stat="ecdf")
 #'
 #' ########### Distance between training data, new data and test data (here Chile):
 #' plot(splotdata[,"Country"])
@@ -48,7 +53,10 @@
 #' ########### Distance between training data, new data and CV folds:
 #' folds <- createFolds(1:nrow(splotdata), k=3, returnTrain=FALSE)
 #' dist <- geodist(x=splotdata, modeldomain=studyArea, cvfolds=folds)
+#' # Using density functions
 #' plot(dist)
+#' # Using ECDFs (relevant for nndm and knnmd methods)
+#' plot(dist, stat="ecdf")
 #'
 #' ########### Distances in the feature space:
 #' predictors <- terra::rast(system.file("extdata","predictors_chile.tif", package="CAST"))
@@ -65,6 +73,22 @@
 #'                 variables=c("bio_1","bio_12", "elev"))
 #' plot(dist)
 #'
+#'############Distances in temporal space
+#' library(lubridate)
+#' library(ggplot2)
+#' data(cookfarm)
+#' dat <- st_as_sf(cookfarm,coords=c("Easting","Northing"))
+#' st_crs(dat) <- 26911
+#' trainDat <- dat[dat$altitude==-0.3&lubridate::year(dat$Date)==2010,]
+#' predictionDat <- dat[dat$altitude==-0.3&lubridate::year(dat$Date)==2011,]
+#' trainDat$week <- lubridate::week(trainDat$Date)
+#' cvfolds <- CreateSpacetimeFolds(trainDat,timevar = "week")
+#'
+#' dist <- geodist(trainDat,preddata = predictionDat,cvfolds = cvfolds$indexOut,
+#'    type="time",time_unit="days")
+#' plot(dist)+ xlim(0,10)
+#'
+#'
 #' ############ Example for a random global dataset
 #' ############ (refer to figure in Meyer and Pebesma 2022)
 #'
@@ -95,7 +119,7 @@
 #' @export
 
 geodist <- function(x,
-                    modeldomain,
+                    modeldomain=NULL,
                     type = "geo",
                     cvfolds=NULL,
                     cvtrain=NULL,
@@ -103,14 +127,15 @@ geodist <- function(x,
                     preddata=NULL,
                     samplesize=2000,
                     sampling = "regular",
-                    variables=NULL){
-
+                    variables=NULL,
+                    timevar=NULL,
+                    time_unit="auto"){
 
   # input formatting ------------
+  if(is.null(modeldomain)&!is.null(preddata)){
+    modeldomain <- sf::st_bbox(preddata)
+  }
   if (inherits(modeldomain, "Raster")) {
-    #    if (!requireNamespace("raster", quietly = TRUE))
-    #      stop("package raster required: install that first")
-    message("Raster will soon not longer be supported. Use terra or stars instead")
     modeldomain <- methods::as(modeldomain,"SpatRaster")
   }
   if (inherits(modeldomain, "stars")) {
@@ -122,7 +147,6 @@ geodist <- function(x,
 
 
 
-  x <- sf::st_transform(x,4326)
   if(type == "feature"){
     if(is.null(variables)){
       variables <- names(modeldomain)
@@ -134,16 +158,13 @@ geodist <- function(x,
       if(class(x)[1]=="sfc_POINT"){
         x <- sf::st_as_sf(x)
       }
-
-      #x <- sf::st_as_sf(raster::extract(modeldomain, x, df = TRUE, sp = TRUE))
-      x <- sf::st_as_sf(terra::extract(modeldomain, x, na.rm=FALSE,bind=TRUE))
-      x <- sf::st_transform(x,4326)
+      x <- sf::st_as_sf(terra::extract(modeldomain, terra::vect(x), na.rm=FALSE,bind=TRUE))
     }
+    x <- sf::st_transform(x,4326)
     if(!is.null(testdata)){
       if(any(!variables%in%names(testdata))){# extract variable values of raster:
         testdata <- sf::st_transform(testdata,sf::st_crs(modeldomain))
-        #testdata <- sf::st_as_sf(raster::extract(modeldomain, testdata, df = TRUE, sp = TRUE))
-        testdata <- sf::st_as_sf(terra::extract(modeldomain, testdata, na.rm=FALSE,bind=TRUE))
+        testdata <- sf::st_as_sf(terra::extract(modeldomain, terra::vect(testdata), na.rm=FALSE,bind=TRUE))
 
         if(any(is.na(testdata))){
           testdata <- na.omit(testdata)
@@ -156,8 +177,7 @@ geodist <- function(x,
     if(!is.null(preddata)){
       if(any(!variables%in%names(preddata))){# extract variable values of raster:
         preddata <- sf::st_transform(preddata,sf::st_crs(modeldomain))
-        #preddata <- sf::st_as_sf(raster::extract(modeldomain, preddata, df = TRUE, sp = TRUE))
-        preddata <- sf::st_as_sf(terra::extract(modeldomain, preddata, na.rm=FALSE,bind=TRUE))
+        preddata <- sf::st_as_sf(terra::extract(modeldomain, terra::vect(preddata), na.rm=FALSE,bind=TRUE))
 
         if(any(is.na(preddata))){
           preddata <- na.omit(preddata)
@@ -167,41 +187,67 @@ geodist <- function(x,
         preddata <- sf::st_transform(preddata,4326)
       }
     }
+    # get names of categorical variables
+    catVars <- names(x[,variables])[which(sapply(x[,variables], class)%in%c("factor","character"))]
+    if(length(catVars)==0) {
+      catVars <- NULL
+    }
+    if(!is.null(catVars)) {
+      message(paste0("variable(s) '", catVars, "' is (are) treated as categorical variables"))
+    }
   }
+  if(type != "feature") {
+    x <- sf::st_transform(x,4326)
+    catVars <- NULL
+  }
+  if (type=="time" & is.null(timevar)){
+    timevar <- names(which(sapply(x, lubridate::is.Date)))
+    message("time variable that has been selected: ",timevar)
+  }
+  if (type=="time"&time_unit=="auto"){
+    time_unit <- units(difftime(sf::st_drop_geometry(x)[,timevar],
+                                sf::st_drop_geometry(x)[,timevar]))
+  }
+
 
 
   # required steps ----
 
   ## Sample prediction location from the study area if preddata not available:
   if(is.null(preddata)){
-    modeldomain <- sampleFromArea(modeldomain, samplesize, type,variables,sampling)}
-  else{
+    modeldomain <- sampleFromArea(modeldomain, samplesize, type,variables,sampling, catVars)
+    } else{
     modeldomain <- preddata
   }
 
   # always do sample-to-sample and sample-to-prediction
-  s2s <- sample2sample(x, type,variables)
-  s2p <- sample2prediction(x, modeldomain, type, samplesize,variables)
+  s2s <- sample2sample(x, type,variables,time_unit,timevar, catVars)
+  s2p <- sample2prediction(x, modeldomain, type, samplesize,variables,time_unit,timevar, catVars)
 
   dists <- rbind(s2s, s2p)
 
   # optional steps ----
   ##### Distance to test data:
   if(!is.null(testdata)){
-    s2t <- sample2test(x, testdata, type,variables)
+    s2t <- sample2test(x, testdata, type,variables,time_unit,timevar, catVars)
     dists <- rbind(dists, s2t)
   }
 
   ##### Distance to CV data:
   if(!is.null(cvfolds)){
-    cvd <- cvdistance(x, cvfolds, cvtrain, type, variables)
+
+    cvd <- cvdistance(x, cvfolds, cvtrain, type, variables,time_unit,timevar, catVars)
     dists <- rbind(dists, cvd)
   }
   class(dists) <- c("geodist", class(dists))
   attr(dists, "type") <- type
 
+  if(type=="time"){
+    attr(dists, "unit") <- time_unit
+  }
+
+
   ##### Compute W statistics
-  #  if(type == "geo"){ # take this condition out once tested for feature space as well
   W_sample <- twosamples::wass_stat(dists[dists$what == "sample-to-sample", "dist"],
                                     dists[dists$what == "prediction-to-sample", "dist"])
   attr(dists, "W_sample") <- W_sample
@@ -215,7 +261,6 @@ geodist <- function(x,
                                   dists[dists$what == "prediction-to-sample", "dist"])
     attr(dists, "W_CV") <- W_CV
   }
-  #  }
 
   return(dists)
 }
@@ -225,8 +270,7 @@ geodist <- function(x,
 
 # Sample to Sample Distance
 
-sample2sample <- function(x, type,variables){
-
+sample2sample <- function(x, type,variables,time_unit,timevar, catVars){
   if(type == "geo"){
     sf::sf_use_s2(TRUE)
     d <- sf::st_distance(x)
@@ -235,19 +279,32 @@ sample2sample <- function(x, type,variables){
     sampletosample <- data.frame(dist = min_d,
                                  what = factor("sample-to-sample"),
                                  dist_type = "geo")
-
-
   }else if(type == "feature"){
     x <- x[,variables]
     x <- sf::st_drop_geometry(x)
-    scaleparam <- attributes(scale(x))
-    x <- data.frame(scale(x))
-    x_clean <- data.frame(x[complete.cases(x),])
+
+    if(!is.null(catVars)) {
+      x_cat <- x[,catVars,drop=FALSE]
+      x_num <- x[,-which(names(x)%in%catVars),drop=FALSE]
+      scaleparam <- attributes(scale(x_num))
+      x_num <- data.frame(scale(x_num))
+      x <- as.data.frame(cbind(x_num, lapply(x_cat, as.factor)))
+      x_clean <- x[complete.cases(x),]
+    } else {
+      scaleparam <- attributes(scale(x))
+      x <- data.frame(scale(x))
+      x_clean <- data.frame(x[complete.cases(x),])
+    }
+
     # sample to sample feature distance
     d <- c()
     for (i in 1:nrow(x_clean)){
 
-      trainDist <-  FNN::knnx.dist(x_clean[i,],x_clean,k=1)
+      if(is.null(catVars)) {
+        trainDist <-  FNN::knnx.dist(x_clean[i,],x_clean,k=1)
+      } else {
+        trainDist <- gower::gower_dist(x_clean[i,],x_clean)
+      }
 
       trainDist[i] <- NA
       d <- c(d,min(trainDist,na.rm=T))
@@ -256,13 +313,25 @@ sample2sample <- function(x, type,variables){
                                  what = factor("sample-to-sample"),
                                  dist_type = "feature")
 
+  }else if(type == "time"){ # calculate temporal distance matrix
+    d <- matrix(ncol=nrow(x),nrow=nrow(x))
+    for (i in 1:nrow(x)){
+      d[i,] <- abs(difftime(sf::st_drop_geometry(x)[,timevar],
+                            sf::st_drop_geometry(x)[i,timevar],
+                            units=time_unit))
+    }
+    diag(d) <- Inf
+    min_d <- apply(d, 1, min)
+    sampletosample <- data.frame(dist = min_d,
+                                 what = factor("sample-to-sample"),
+                                 dist_type = "time")
   }
   return(sampletosample)
 }
 
 
 # Sample to Prediction
-sample2prediction = function(x, modeldomain, type, samplesize,variables){
+sample2prediction = function(x, modeldomain, type, samplesize,variables,time_unit,timevar, catVars){
 
   if(type == "geo"){
     modeldomain <- sf::st_transform(modeldomain, sf::st_crs(x))
@@ -276,24 +345,62 @@ sample2prediction = function(x, modeldomain, type, samplesize,variables){
   }else if(type == "feature"){
     x <- x[,variables]
     x <- sf::st_drop_geometry(x)
-    scaleparam <- attributes(scale(x))
-    x <- data.frame(scale(x))
-    x_clean <- x[complete.cases(x),]
-
     modeldomain <- modeldomain[,variables]
     modeldomain <- sf::st_drop_geometry(modeldomain)
-    modeldomain <- data.frame(scale(modeldomain,center=scaleparam$`scaled:center`,
-                                    scale=scaleparam$`scaled:scale`))
+
+    if(!is.null(catVars)) {
+
+      x_cat <- x[,catVars,drop=FALSE]
+      x_num <- x[,-which(names(x)%in%catVars),drop=FALSE]
+      scaleparam <- attributes(scale(x_num))
+      x_num <- data.frame(scale(x_num))
+
+      modeldomain_num <- modeldomain[,-which(names(modeldomain)%in%catVars),drop=FALSE]
+      modeldomain_cat <- modeldomain[,catVars,drop=FALSE]
+      modeldomain_num <- data.frame(scale(modeldomain_num,center=scaleparam$`scaled:center`,
+                                      scale=scaleparam$`scaled:scale`))
+
+      x <- as.data.frame(cbind(x_num, lapply(x_cat, as.factor)))
+      x_clean <- x[complete.cases(x),]
+      modeldomain <- as.data.frame(cbind(modeldomain_num, lapply(modeldomain_cat, as.factor)))
+
+    } else {
+      scaleparam <- attributes(scale(x))
+      x <- data.frame(scale(x))
+      x_clean <- x[complete.cases(x),]
+
+      modeldomain <- data.frame(scale(modeldomain,center=scaleparam$`scaled:center`,
+                                      scale=scaleparam$`scaled:scale`))
+    }
+
 
     target_dist_feature <- c()
     for (i in 1:nrow(modeldomain)){
 
-      trainDist <-  FNN::knnx.dist(modeldomain[i,],x_clean,k=1)
+      if(is.null(catVars)) {
+        trainDist <-  FNN::knnx.dist(modeldomain[i,],x_clean,k=1)
+      } else {
+        trainDist <- gower::gower_dist(modeldomain[i,], x_clean)
+      }
+
       target_dist_feature <- c(target_dist_feature,min(trainDist,na.rm=T))
     }
     sampletoprediction <- data.frame(dist = target_dist_feature,
                                      what = "prediction-to-sample",
                                      dist_type = "feature")
+  }else if(type == "time"){
+
+    min_d0 <- c()
+    for (i in 1:nrow(modeldomain)){
+      min_d0[i] <- min(abs(difftime(sf::st_drop_geometry(modeldomain)[i,timevar],
+                                    sf::st_drop_geometry(x)[,timevar],
+                                    units=time_unit)))
+    }
+
+    sampletoprediction <- data.frame(dist = min_d0,
+                                     what = factor("prediction-to-sample"),
+                                     dist_type = "time")
+
   }
 
   return(sampletoprediction)
@@ -303,7 +410,7 @@ sample2prediction = function(x, modeldomain, type, samplesize,variables){
 # sample to test
 
 
-sample2test <- function(x, testdata, type,variables){
+sample2test <- function(x, testdata, type,variables,time_unit,timevar, catVars){
 
   if(type == "geo"){
     testdata <- sf::st_transform(testdata,4326)
@@ -316,26 +423,64 @@ sample2test <- function(x, testdata, type,variables){
 
 
   }else if(type == "feature"){
+
+
     x <- x[,variables]
     x <- sf::st_drop_geometry(x)
-    scaleparam <- attributes(scale(x))
-    x <- data.frame(scale(x))
-    x_clean <- x[complete.cases(x),]
     testdata <- testdata[,variables]
     testdata <- sf::st_drop_geometry(testdata)
-    testdata <- data.frame(scale(testdata,center=scaleparam$`scaled:center`,
-                                 scale=scaleparam$`scaled:scale`))
+
+    if(!is.null(catVars)) {
+
+      x_cat <- x[,catVars,drop=FALSE]
+      x_num <- x[,-which(names(x)%in%catVars),drop=FALSE]
+      scaleparam <- attributes(scale(x_num))
+      x_num <- data.frame(scale(x_num))
+
+      testdata_num <- testdata[,-which(names(testdata)%in%catVars),drop=FALSE]
+      testdata_cat <- testdata[,catVars,drop=FALSE]
+      testdata_num <- data.frame(scale(testdata_num,center=scaleparam$`scaled:center`,
+                                          scale=scaleparam$`scaled:scale`))
+
+      x <- as.data.frame(cbind(x_num, lapply(x_cat, as.factor)))
+      x_clean <- x[complete.cases(x),]
+      testdata <- as.data.frame(cbind(testdata_num, lapply(testdata_cat, as.factor)))
+
+    } else {
+      scaleparam <- attributes(scale(x))
+      x <- data.frame(scale(x))
+      x_clean <- x[complete.cases(x),]
+
+      testdata <- data.frame(scale(testdata,center=scaleparam$`scaled:center`,
+                                      scale=scaleparam$`scaled:scale`))
+    }
 
 
     test_dist_feature <- c()
     for (i in 1:nrow(testdata)){
 
-      testDist <- FNN::knnx.dist(testdata[i,],x_clean,k=1)
+      if(is.null(catVars)) {
+        testDist <- FNN::knnx.dist(testdata[i,],x_clean,k=1)
+      } else {
+        testDist <- gower::gower_dist(testdata[i,], x_clean)
+      }
       test_dist_feature <- c(test_dist_feature,min(testDist,na.rm=T))
     }
     dists_test <- data.frame(dist = test_dist_feature,
                              what = "test-to-sample",
                              dist_type = "feature")
+  }else if (type=="time"){
+    min_d0 <- c()
+    for (i in 1:nrow(testdata)){
+      min_d0[i] <- min(abs(difftime(sf::st_drop_geometry(testdata)[i,timevar],
+                                    sf::st_drop_geometry(x)[,timevar],
+                                    units=time_unit)))
+    }
+
+    dists_test <- data.frame(dist = min_d0,
+                             what = factor("test-to-sample"),
+                             dist_type = "time")
+
 
 
   }
@@ -346,7 +491,7 @@ sample2test <- function(x, testdata, type,variables){
 
 # between folds
 
-cvdistance <- function(x, cvfolds, cvtrain, type, variables){
+cvdistance <- function(x, cvfolds, cvtrain, type, variables,time_unit,timevar, catVars){
 
   if(!is.null(cvfolds)&!is.list(cvfolds)){ # restructure input if CVtest only contains the fold ID
     tmp <- list()
@@ -377,8 +522,16 @@ cvdistance <- function(x, cvfolds, cvtrain, type, variables){
   }else if(type == "feature"){
     x <- x[,variables]
     x <- sf::st_drop_geometry(x)
-    x <- data.frame(scale(x))
 
+    if(is.null(catVars)) {
+      x <- data.frame(scale(x))
+    } else {
+      x_cat <- x[,catVars,drop=FALSE]
+      x_num <- x[,-which(names(x)%in%catVars),drop=FALSE]
+      scaleparam <- attributes(scale(x_num))
+      x_num <- data.frame(scale(x_num))
+      x <- as.data.frame(cbind(x_num, lapply(x_cat, as.factor)))
+    }
 
     d_cv <- c()
     for(i in 1:length(cvfolds)){
@@ -396,25 +549,61 @@ cvdistance <- function(x, cvfolds, cvtrain, type, variables){
 
       for (k in 1:nrow(testdata_i)){
 
-        trainDist <-  tryCatch(FNN::knnx.dist(testdata_i[k,],traindata_i,k=1),
-                               error = function(e)e)
-        if(inherits(trainDist, "error")){
-          trainDist <- NA
-          message("warning: no distance could be calculated for a fold.
+        if(is.null(catVars)) {
+          trainDist <-  tryCatch(FNN::knnx.dist(testdata_i[k,],traindata_i,k=1),
+                                 error = function(e)e)
+          if(inherits(trainDist, "error")){
+            trainDist <- NA
+            message("warning: no distance could be calculated for a fold.
                   Possibly because predictor values are NA")
+          }
+        } else {
+          trainDist <-  tryCatch(gower::gower_dist(testdata_i[i,], traindata_i),
+                                 error = function(e)e)
+          if(inherits(trainDist, "error")){
+            trainDist <- NA
+            message("warning: no distance could be calculated for a fold.
+                  Possibly because predictor values are NA")
+          }
         }
 
+
         trainDist[k] <- NA
         d_cv <- c(d_cv,min(trainDist,na.rm=T))
       }
     }
 
-
     dists_cv <- data.frame(dist = d_cv,
                            what = factor("CV-distances"),
                            dist_type = "feature")
 
+  }else if(type == "time"){
+    d_cv <- c()
+    d_cv_tmp <- c()
+    for (i in 1:length(cvfolds)){
+      if(!is.null(cvtrain)){
+        for (k in 1:length(cvfolds[[i]])){
+          d_cv_tmp[k] <- min(abs(difftime(sf::st_drop_geometry(x)[cvfolds[[i]][k],timevar],
+                                          sf::st_drop_geometry(x)[cvtrain[[i]],timevar],
+                                          units=time_unit)))
+        }
+      }else{
+        for (k in 1:length(cvfolds[[i]])){
+          d_cv_tmp[k] <- min(abs(difftime(sf::st_drop_geometry(x)[cvfolds[[i]][k],timevar],
+                                          sf::st_drop_geometry(x)[-cvfolds[[i]],timevar],
+                                          units=time_unit)))
+        }
+      }
+      d_cv <- c(d_cv,d_cv_tmp)
+    }
+
+
+    dists_cv <- data.frame(dist = d_cv,
+                           what = factor("CV-distances"),
+                           dist_type = "time")
+
   }
+
   return(dists_cv)
 }
 
@@ -422,7 +611,7 @@ cvdistance <- function(x, cvfolds, cvtrain, type, variables){
 
 
 
-sampleFromArea <- function(modeldomain, samplesize, type,variables,sampling){
+sampleFromArea <- function(modeldomain, samplesize, type,variables,sampling, catVars){
 
   ##### Distance to prediction locations:
   # regularly spread points (prediction locations):
@@ -460,7 +649,13 @@ sampleFromArea <- function(modeldomain, samplesize, type,variables,sampling){
 
 
   if(type == "feature"){
-    modeldomain <- terra::project(modeldomain, "epsg:4326")
+
+    if(is.null(catVars)) {
+      modeldomain <- terra::project(modeldomain, "epsg:4326")
+    } else {
+      modeldomain <- terra::project(modeldomain, "epsg:4326", method="near")
+    }
+
     predictionloc <- sf::st_as_sf(terra::extract(modeldomain,terra::vect(predictionloc),bind=TRUE))
     predictionloc <- na.omit(predictionloc)
   }
