@@ -30,6 +30,7 @@ plot.trainDI = function(x, ...){
 #'
 #' @param x aoa object
 #' @param samplesize numeric. How many prediction samples should be plotted?
+#' @param variable character. Variable for which to generate the density plot. 'DI' or 'LPD'
 #' @param ... other params
 #'
 #' @import ggplot2
@@ -38,56 +39,95 @@ plot.trainDI = function(x, ...){
 #'
 #' @export
 
-plot.aoa = function(x, samplesize = 1000, ...){
+plot.aoa = function(x, samplesize = 1000, variable = "DI", ...){
 
+  if (variable == "DI") {
+    trainDI = data.frame(DI = x$parameters$trainDI,
+                         what = "trainDI")
 
-  trainDI = data.frame(DI = x$parameters$trainDI,
-                       what = "trainDI")
+    if(inherits(x$AOA, "RasterLayer")){
+      targetDI = terra::spatSample(methods::as(x$DI, "SpatRaster"),
+                                   size = samplesize, method = "regular")
+      targetDI = data.frame(DI = as.numeric(targetDI[, 1]),
+                            what = "predictionDI")
+    }else if(inherits(x$AOA, "stars")){
+      targetDI = terra::spatSample(methods::as(x$DI, "SpatRaster"),
+                                   size = samplesize, method = "regular")
+      targetDI = data.frame(DI = as.numeric(targetDI[, 1]),
+                            what = "predictionDI")
+    }else if(inherits(x$AOA, "SpatRaster")){
+      targetDI = terra::spatSample(x$DI, size = samplesize, method = "regular")
+      targetDI = data.frame(DI = as.numeric(targetDI[, 1]),
+                            what = "predictionDI")
+    }else{
+      targetDI = data.frame(DI = sample(x$DI, size = samplesize),
+                            what = "predictionDI")
+    }
 
+    dfDI = rbind(trainDI, targetDI)
 
-
-  if(inherits(x$AOA, "RasterLayer")){
-    targetDI = terra::spatSample(methods::as(x$DI, "SpatRaster"),
-                                 size = samplesize, method="regular")
-    targetDI = data.frame(DI = as.numeric(targetDI[,1]),
-                          what = "predictionDI")
-  }else if(inherits(x$AOA, "stars")){
-    targetDI = terra::spatSample(methods::as(x$DI, "SpatRaster"),
-                                 size = samplesize,method="regular")
-    targetDI = data.frame(DI = as.numeric(targetDI[,1]),
-                          what = "predictionDI")
-  }else if(inherits(x$AOA, "SpatRaster")){
-    targetDI = terra::spatSample(x$DI, size = samplesize,method="regular")
-    targetDI = data.frame(DI = as.numeric(targetDI[,1]),
-                          what = "predictionDI")
-  }else{
-    targetDI = data.frame(DI = sample(x$DI, size = samplesize),
-                          what = "predictionDI")
+    plot = ggplot(dfDI, aes_string(x = "DI", group = "what", fill = "what"))+
+      geom_density(adjust=1.5, alpha=.4)+
+      scale_fill_discrete(name = "Set")+
+      geom_vline(aes(xintercept = x$parameters$threshold, linetype = "AOA_threshold"))+
+      scale_linetype_manual(name = "", values = c(AOA_threshold = "dashed"))+
+      theme_bw()+
+      theme(legend.position = "bottom")
   }
 
 
-
-  dfDI = rbind(trainDI, targetDI)
+  if (variable == "LPD") {
+    trainLPD = data.frame(LPD = x$parameters$trainLPD,
+                          what = "trainLPD")
 
 
-  ggplot(dfDI, aes_string(x = "DI", group = "what", fill = "what"))+
-    geom_density(adjust=1.5, alpha=.4)+
-    scale_fill_discrete(name = "Set")+
-    geom_vline(aes(xintercept = x$parameters$threshold, linetype = "AOA_threshold"))+
-    scale_linetype_manual(name = "", values = c(AOA_threshold = "dashed"))+
-    theme_bw()+
-    theme(legend.position = "bottom")
+
+    if(inherits(x$AOA, "RasterLayer")){
+      targetLPD = terra::spatSample(methods::as(x$LPD, "SpatRaster"),
+                                    size = samplesize, method = "regular")
+      targetLPD = data.frame(LPD = as.numeric(targetLPD[, 1]),
+                             what = "predictionLPD")
+    }else if(inherits(x$AOA, "stars")){
+      targetLPD = terra::spatSample(methods::as(x$LPD, "SpatRaster"),
+                                    size = samplesize, method = "regular")
+      targetLPD = data.frame(LPD = as.numeric(targetLPD[, 1]),
+                             what = "predictionLPD")
+    }else if(inherits(x$AOA, "SpatRaster")){
+      targetLPD = terra::spatSample(x$LPD, size = samplesize, method = "regular")
+      targetLPD = data.frame(LPD = as.numeric(targetLPD[, 1]),
+                             what = "predictionLPD")
+    }else{
+      targetLPD = data.frame(LPD = sample(x$LPD, size = samplesize),
+                             what = "predictionLPD")
+    }
+
+    dfLPD = rbind(trainLPD, targetLPD)
+
+
+    plot = ggplot(dfLPD, aes_string(x = "LPD", group = "what", fill = "what"))+
+      geom_density(adjust=1.5, alpha=0.4)+
+      scale_fill_discrete(name = "Set")+
+      geom_vline(aes(xintercept = median(x$parameters$trainLPD), linetype = "MtrainLPD"))+
+      scale_linetype_manual(name = "", values = c(MtrainLPD = "dashed"))+
+      theme_bw()+
+      theme(legend.position = "bottom")
+
+  }
+
+  return(plot)
 }
 
 
 
 #' @name plot
 #' @param x An object of type \emph{nndm}.
+#' @param type String, defaults to "strict" to show the original nearest neighbour distance definitions in the legend.
+#' Alternatively, set to "simple" to have more intuitive labels.
 #' @param ... other arguments.
 #' @author Carles Milà
 #'
 #' @export
-plot.nndm <- function(x, ...){
+plot.nndm <- function(x, type="strict", stat = "ecdf", ...){
 
   # Prepare data for plotting: Gij function
   Gij_df <- data.frame(r=x$Gij[order(x$Gij)])
@@ -116,38 +156,86 @@ plot.nndm <- function(x, ...){
                                    val=sum(x$Gj<=x$phi)/length(x$Gj)))
   Gj_df$Function <- "3_Gj(r)"
 
-  # Merge data for plotting
-  Gplot <- rbind(Gij_df, Gjstar_df, Gj_df)
+  # Merge data for plotting, get maxdist relevant for plotting
+  if(any(Gj_df$val==1)&any(Gjstar_df$val==1)&any(Gij_df$val==1)){
+    Gplot <- rbind(Gij_df, Gjstar_df, Gj_df)
+    maxdist <- max(Gplot$r[Gplot$val!=1]) + 1e-9
+    Gplot <- Gplot[Gplot$r <= maxdist,]
+    Gplot <- rbind(Gplot, data.frame(r=maxdist, val=1,
+                                     Function = c("1_Gij(r)", "2_Gjstar(r)", "3_Gj(r)")))
+  }else{
+    Gplot <- rbind(Gij_df, Gjstar_df, Gj_df)
+  }
+
+  # Define colours matching those of geodist
+  myColors <- RColorBrewer::brewer.pal(3, "Dark2")
 
   # Plot
-  ggplot2::ggplot(Gplot) +
-    ggplot2::geom_step(ggplot2::aes_string(x="r", y="val", colour="Function", size="Function"),
-                       alpha = 0.8) +
-    ggplot2::scale_size_manual(values=c(1.1, 1.1, 0.5),
-                               labels=c(expression(hat(G)[ij](r)),
-                                        expression(hat(G)[j]^"*"*"(r,"*bold(L)*")"),
-                                        expression(hat(G)[j](r)))) +
-    ggplot2::scale_colour_manual(values=c("#000000", "#E69F00", "#56B4E9"),
-                                 labels=c(expression(hat(G)[ij](r)),
-                                          expression(hat(G)[j]^"*"*"(r,"*bold(L)*")"),
-                                          expression(hat(G)[j](r)))) +
-    ggplot2::ylab(expression(paste(hat(G)[ij](r), ", ",
-                                   hat(G)[j]^"*"*"(r,"*bold(L)*")", ", ",
-                                   hat(G)[j](r)))) +
-    ggplot2::labs(colour="", size="") +
-    ggplot2::theme_bw() +
-    ggplot2::theme(legend.text.align=0,
-                   legend.text=ggplot2::element_text(size=12))
+  if(stat=="ecdf"){
+    p <- ggplot2::ggplot(data=Gplot, ggplot2::aes_string(x="r", group="Function", col="Function")) +
+      ggplot2::geom_vline(xintercept=0, lwd = 0.1) +
+      ggplot2::geom_hline(yintercept=0, lwd = 0.1) +
+      ggplot2::geom_hline(yintercept=1, lwd = 0.1) +
+      ggplot2::stat_ecdf(geom = "step", lwd = 0.8) +
+      ggplot2::theme_bw() +
+      ggplot2::ylab("ECDF") +
+      ggplot2::labs(group="Distance function", col="Distance function") +
+      ggplot2::theme(legend.position = "bottom",
+                     legend.text=ggplot2::element_text(size=10))
+
+    if(type=="strict"){
+      p <-  p +
+        ggplot2::scale_colour_manual(values=c(myColors[2], myColors[3], myColors[1]),
+                                     labels=c(expression(hat(G)[ij](r)),
+                                              expression(hat(G)[j]^"*"*"(r,L)"),
+                                              expression(hat(G)[j](r))))
+    }else if(type == "simple"){
+      p <-  p +
+        ggplot2::scale_colour_manual(values=c(myColors[2], myColors[3], myColors[1]),
+                                     labels=c("prediction-to-sample",
+                                              "CV-distances",
+                                              "sample-to-sample"))
+    }
+
+  }else if(stat=="density"){
+    p <- ggplot2::ggplot(data=Gplot, ggplot2::aes_string(x="r", group="Function", fill="Function")) +
+      ggplot2::geom_density(adjust=1.5, alpha=.5, stat=stat, lwd = 0.3) +
+      ggplot2::theme_bw() +
+      ggplot2::ylab("Density") +
+      ggplot2::labs(group="Distance function", col="Distance function") +
+      ggplot2::theme(legend.position = "bottom",
+                     legend.text=ggplot2::element_text(size=10))
+
+    if(type=="strict"){
+      p <-  p +
+        ggplot2::scale_fill_manual(values=c(myColors[2], myColors[3], myColors[1]),
+                                   labels=c(expression(hat(G)[ij](r)),
+                                            expression(hat(G)[j]^"*"*"(r,L)"),
+                                            expression(hat(G)[j](r))))
+    }else if(type == "simple"){
+      p <-  p +
+        ggplot2::scale_fill_manual(values=c(myColors[2], myColors[3], myColors[1]),
+                                   labels=c("prediction-to-sample",
+                                            "CV-distances",
+                                            "sample-to-sample"))
+    }
+  }
+
+  p
+
 }
 
 
 #' @name plot
 #' @param x An object of type \emph{knndm}.
+#' @param type String, defaults to "strict" to show the original nearest neighbour distance definitions in the legend.
+#' Alternatively, set to "simple" to have more intuitive labels.
+#' @param stat String, defaults to "ecdf" but can be set to "density" to estimate density functions.
 #' @param ... other arguments.
 #' @author Carles Milà
 #'
 #' @export
-plot.knndm <- function(x, ...){
+plot.knndm <- function(x, type="strict", stat = "ecdf", ...){
 
   # Prepare data for plotting: Gij function
   Gij_df <- data.frame(r=x$Gij[order(x$Gij)])
@@ -164,19 +252,61 @@ plot.knndm <- function(x, ...){
   # Merge data for plotting
   Gplot <- rbind(Gij_df, Gjstar_df, Gj_df)
 
+  # Define colours matching those of geodist
+  myColors <- RColorBrewer::brewer.pal(3, "Dark2")
+
   # Plot
-  ggplot2::ggplot(data=Gplot, ggplot2::aes_string(x="r", group="Function", col="Function")) +
-    ggplot2::geom_vline(xintercept=0, lwd = 0.1) +
-    ggplot2::geom_hline(yintercept=0, lwd = 0.1) +
-    ggplot2::geom_hline(yintercept=1, lwd = 0.1) +
-    ggplot2::stat_ecdf(geom = "step", lwd = 1) +
-    ggplot2::scale_colour_manual(values=c("#000000", "#E69F00", "#56B4E9"),
-                                 labels=c(expression(hat(G)[ij](r)),
-                                          expression(hat(G)[j]^"*"*"(r,L)"),
-                                          expression(hat(G)[j](r)))) +
-    ggplot2::ylab(expression(paste(hat(G)[ij](r), ", ",
-                                   hat(G)[j]^"*"*"(r,L)", ", ",
-                                   hat(G)[j](r))))
+  if(stat=="ecdf"){
+    p <- ggplot2::ggplot(data=Gplot, ggplot2::aes_string(x="r", group="Function", col="Function")) +
+      ggplot2::geom_vline(xintercept=0, lwd = 0.1) +
+      ggplot2::geom_hline(yintercept=0, lwd = 0.1) +
+      ggplot2::geom_hline(yintercept=1, lwd = 0.1) +
+      ggplot2::stat_ecdf(geom = "step", lwd = 0.8) +
+      ggplot2::theme_bw() +
+      ggplot2::ylab("ECDF") +
+      ggplot2::labs(group="Distance function", col="Distance function") +
+      ggplot2::theme(legend.position = "bottom",
+                     legend.text=ggplot2::element_text(size=10))
+
+    if(type=="strict"){
+      p <-  p +
+        ggplot2::scale_colour_manual(values=c(myColors[2], myColors[3], myColors[1]),
+                                     labels=c(expression(hat(G)[ij](r)),
+                                              expression(hat(G)[j]^"*"*"(r,L)"),
+                                              expression(hat(G)[j](r))))
+    }else if(type == "simple"){
+      p <-  p +
+        ggplot2::scale_colour_manual(values=c(myColors[2], myColors[3], myColors[1]),
+                                     labels=c("prediction-to-sample",
+                                              "CV-distances",
+                                              "sample-to-sample"))
+    }
+
+  }else if(stat=="density"){
+    p <- ggplot2::ggplot(data=Gplot, ggplot2::aes_string(x="r", group="Function", fill="Function")) +
+      ggplot2::geom_density(adjust=1.5, alpha=.5, stat=stat, lwd = 0.3) +
+      ggplot2::theme_bw() +
+      ggplot2::ylab("Density") +
+      ggplot2::labs(group="Distance function", col="Distance function") +
+      ggplot2::theme(legend.position = "bottom",
+                     legend.text=ggplot2::element_text(size=10))
+
+    if(type=="strict"){
+      p <-  p +
+        ggplot2::scale_fill_manual(values=c(myColors[2], myColors[3], myColors[1]),
+                                   labels=c(expression(hat(G)[ij](r)),
+                                            expression(hat(G)[j]^"*"*"(r,L)"),
+                                            expression(hat(G)[j](r))))
+    }else if(type == "simple"){
+      p <-  p +
+        ggplot2::scale_fill_manual(values=c(myColors[2], myColors[3], myColors[1]),
+                                   labels=c("prediction-to-sample",
+                                            "CV-distances",
+                                            "sample-to-sample"))
+    }
+  }
+
+  p
 }
 
 #' Plot results of a Forward feature selection or best subset selection
@@ -196,8 +326,7 @@ plot.knndm <- function(x, ...){
 #' @param lwd Numeric. Width of the error bars
 #' @param pch Numeric. Type of point marking the best models
 #' @param ... Further arguments for base plot if type="selected"
-#' @author Marvin Ludwig and Hanna Meyer
-#' @seealso \code{\link{ffs}}, \code{\link{bss}}
+#' @author Marvin Ludwig, Hanna Meyer
 #' @examples
 #' \dontrun{
 #' data(splotdata)
@@ -283,7 +412,7 @@ plot.ffs <- function(x,plotType="all",palette=rainbow,reverse=FALSE,
         ggplot2::geom_point(ggplot2::aes_string(colour="nvar"),size=size)+
         ggplot2::geom_point(data=output_df[bestmodels, ],
                             ggplot2::aes_string(x = "run", y = "value"),
-                            pch=pch,colour=marker,lwd=size)+
+                            pch=pch,colour=marker,size=size)+
         ggplot2::scale_x_continuous(name = "Model run", breaks = pretty(output_df$run))+
         ggplot2::scale_y_continuous(name = metric)+
         ggplot2::scale_colour_gradientn(breaks=seq(2,max(output_df$nvar),
@@ -298,7 +427,7 @@ plot.ffs <- function(x,plotType="all",palette=rainbow,reverse=FALSE,
         ggplot2::geom_point(ggplot2::aes_string(colour="nvar"),size=size)+
         ggplot2::geom_point(data=output_df[bestmodels, ],
                             ggplot2::aes_string(x = "run", y = "value"),
-                            pch=pch,colour=marker,lwd=size)+
+                            pch=pch,colour=marker,size=size)+
         ggplot2::scale_x_continuous(name = "Model run", breaks = pretty(dfint$run))+
         ggplot2::scale_y_continuous(name = metric)+
         ggplot2::scale_colour_manual(values = cols, name = "variables")
@@ -311,7 +440,8 @@ plot.ffs <- function(x,plotType="all",palette=rainbow,reverse=FALSE,
 
 
 #' @name plot
-#' @description Density plot of nearest neighbor distances in geographic space or feature space between training data as well as between training data and prediction locations.
+#' @description Density plot of nearest neighbor distances in geographic space or feature space between training data as well as between training data and
+#' prediction locations.
 #' Optional, the nearest neighbor distances between training data and test data or between training data and CV iterations is shown.
 #' The plot can be used to check the suitability of a chosen CV method to be representative to estimate map accuracy.
 #' @param x geodist, see \code{\link{geodist}}
@@ -325,6 +455,14 @@ plot.ffs <- function(x,plotType="all",palette=rainbow,reverse=FALSE,
 
 plot.geodist <- function(x, unit = "m", stat = "density", ...){
 
+  # Define colours - they must match those of knndm and nndm
+  labs <- c("sample-to-sample",
+            "prediction-to-sample",
+            "CV-distances",
+            "test-to-sample")
+  myColors <- RColorBrewer::brewer.pal(4, "Dark2")
+  names(myColors) <- labs
+
 
   type <- attr(x, "type")
 
@@ -338,10 +476,14 @@ plot.geodist <- function(x, unit = "m", stat = "density", ...){
   if( type=="feature"){ xlabs <- "feature space distances"}
   what <- "" #just to avoid check note
   if (type=="feature"){unit ="unitless"}
+
+  if (type=="time"){unit = attr(x,"unit")}
+  if( type=="time"){ xlabs <- paste0("temporal distances (",unit,")")}
   if(stat=="density"){
     p <- ggplot2::ggplot(data=x, aes(x=dist, group=what, fill=what)) +
-      ggplot2::geom_density(adjust=1.5, alpha=.4, stat=stat) +
-      ggplot2::scale_fill_discrete(name = "distance function") +
+      ggplot2::geom_density(adjust=1.5, alpha=.5, stat=stat, lwd = 0.3) +
+      ggplot2::scale_fill_manual(name = "distance function", values = myColors) +
+      ggplot2::theme_bw() +
       ggplot2::xlab(xlabs) +
       ggplot2::theme(legend.position="bottom",
                      plot.margin = unit(c(0,0.5,0,0),"cm"))
@@ -351,7 +493,8 @@ plot.geodist <- function(x, unit = "m", stat = "density", ...){
       ggplot2::geom_hline(yintercept=0, lwd = 0.1) +
       ggplot2::geom_hline(yintercept=1, lwd = 0.1) +
       ggplot2::stat_ecdf(geom = "step", lwd = 1) +
-      ggplot2::scale_color_discrete(name = "distance function") +
+      ggplot2::scale_color_manual(name = "distance function", values = myColors) +
+      ggplot2::theme_bw() +
       ggplot2::xlab(xlabs) +
       ggplot2::ylab("ECDF") +
       ggplot2::theme(legend.position="bottom",
@@ -362,7 +505,7 @@ plot.geodist <- function(x, unit = "m", stat = "density", ...){
 
 
 #' @name plot
-#' @description Plot the DI and errormetric from Cross-Validation with the modelled relationship
+#' @description Plot the DI/LPD and errormetric from Cross-Validation with the modeled relationship
 #' @param x errorModel, see \code{\link{DItoErrormetric}}
 #' @param ... other params
 #' @export
@@ -372,26 +515,22 @@ plot.geodist <- function(x, unit = "m", stat = "density", ...){
 
 plot.errorModel <- function(x, ...){
 
-  performance = attr(x, "performance")[,c("DI", "metric")]
+  variable = attr(x, "variable")
+  metric = attr(x, "metric")
+
+  performance = attr(x, "performance")[,c(variable, "metric")]
   performance$what = "cross-validation"
 
-  model_line = data.frame(DI = performance$DI,
+  model_line = data.frame(variable = performance[, variable],
                           metric = predict(x, performance),
                           what = "model")
 
-
   p = ggplot()+
-    geom_point(data = performance, mapping = aes_string(x = "DI", y = "metric", shape = "what"))+
-    geom_line(data = model_line, mapping =  aes_string(x = "DI", y = "metric", linetype = "what"), lwd = 1)+
+    geom_point(data = performance, mapping = aes_string(x = variable, y = "metric", shape = "what"))+
+    geom_line(data = model_line, mapping =  aes_string(x = "variable", y = "metric", linetype = "what"), lwd = 1)+
+    labs(x = variable, y = metric)+
     theme(legend.title = element_blank(), legend.position = "bottom")
 
   return(p)
 
 }
-
-
-
-
-
-
-
